// Export a function that takes a GraphQL schema as input and returns a string to append to generated TypeScript code
module.exports = {
  plugin: (schema) => {
    // Initialize an empty string to hold the TypeScript code to append
    let appendedString = "";

    // Get the map of all types in the schema
    const typeMap = schema.getTypeMap();

    // Get the fields of the Query type and their names
    const queryFields = typeMap["Query"].getFields();
    const queryNames = Object.keys(typeMap["Query"].getFields()).filter(
      (fieldName) => queryFields[fieldName].args.length > 0 // Only include fields with arguments
    );

    // Get the fields of the Mutation type and their names
    const mutationFields = schema.getTypeMap()["Mutation"].getFields();
    const mutationNames = Object.keys(
      schema.getTypeMap()["Mutation"].getFields()
    ).filter((fieldName) => mutationFields[fieldName].args.length > 0); // Only include fields with arguments

    // Generate TypeScript code for argument types for Query fields, if any exist
    if (queryNames.length) {
      appendedString += "export type QuerysArguments = {\n";

      for (const queryName of queryNames) {
        const queryNameUc =
          queryName.charAt(0).toUpperCase() + queryName.slice(1);
        appendedString += `  ${queryName}: Query${queryNameUc}Args;\n`;
      }

      appendedString += "};\n\n";
      appendedString +=
        "export type QueryArgs<T extends keyof QuerysArguments> = QuerysArguments[T];\n\n";
    }

    // Generate TypeScript code for argument types for Mutation fields, if any exist
    if (mutationNames.length) {
      appendedString += "export type MutationsArguments = {\n";

      for (const mutationName of mutationNames) {
        const mutationNameUc =
          mutationName.charAt(0).toUpperCase() + mutationName.slice(1);
        appendedString += `  ${mutationName}: Mutation${mutationNameUc}Args;\n`;
      }

      appendedString += "};\n\n";
      appendedString +=
        "export type MutationArgs<T extends keyof MutationsArguments> = MutationsArguments[T];\n\n";
    }

    // Return the generated TypeScript code to append to the generated code
    return appendedString;
  },
};
